@@ -62,6 +62,7 @@ struct NautilusDirectoryDetails
 	 */
 	GList *call_when_ready_list;
 	GList *monitor_list;
+	guint call_ready_idle_id;
 
 	NautilusMonitor *monitor;
 	gulong 		 mime_db_monitor;
@@ -104,6 +105,8 @@ struct NautilusDirectoryDetails
 	GnomeVFSAsyncHandle *get_info_in_progress;
 	gboolean get_info_has_slow_mime_type;
 
+	int is_in_trash_state;
+
 	NautilusFile *slow_mime_type_file;
 	GnomeVFSAsyncHandle *slow_mime_type_in_progress;
 
