@@ -73,6 +73,8 @@ path_is_on_readonly_volume (const char *path)
 	GnomeVFSVolume *volume;
 	gboolean res;
 
+	g_assert (path != NULL);
+
 	volume_monitor = gnome_vfs_get_volume_monitor ();
 	volume = gnome_vfs_volume_monitor_get_volume_for_path (volume_monitor, 
 							       path);
@@ -106,6 +108,7 @@ monitor_notify_cb (GnomeVFSMonitorHandle    *handle,
 		nautilus_file_changes_queue_file_changed (info_uri);
 		break;
 	case GNOME_VFS_MONITOR_EVENT_DELETED:
+		nautilus_file_changes_queue_schedule_metadata_remove (info_uri);
 		nautilus_file_changes_queue_file_removed (info_uri);
 		break;
 	case GNOME_VFS_MONITOR_EVENT_CREATED:
