@@ -69,6 +69,16 @@ typedef struct {
 	time_t original_file_mtime;
 } NautilusThumbnailInfo;
 
+struct NautilusThumbnailAsyncLoadHandle {
+	EelReadFileHandle *eel_read_handle;
+	char *file_path;
+	guint base_size;
+	guint nominal_size;
+	gboolean force_nominal; 
+	NautilusThumbnailAsyncLoadFunc load_func;
+	gpointer load_func_user_data;
+};
+
 
 /*
  * Thumbnail thread state.
@@ -87,15 +97,26 @@ static pthread_mutex_t thumbnails_mutex = PTHREAD_MUTEX_INITIALIZER;
    start more than one. Lock thumbnails_mutex when accessing this. */
 static volatile gboolean thumbnail_thread_is_running = FALSE;
 
+/* Added in glib 2.14 */
+#ifndef G_QUEUE_INIT
+#define G_QUEUE_INIT { NULL, NULL, 0 }
+#endif
+
 /* The list of NautilusThumbnailInfo structs containing information about the
    thumbnails we are making. Lock thumbnails_mutex when accessing this. */
-static volatile GList *thumbnails_to_make = NULL;
+static volatile GQueue thumbnails_to_make = G_QUEUE_INIT;
+
+/* Quickly check if uri is in thumbnails_to_make list */
+static GHashTable *thumbnails_to_make_hash = NULL;
+
 /* The currently thumbnailed icon. it also exists in the thumbnails_to_make list
  * to avoid adding it again. Lock thumbnails_mutex when accessing this. */
 static NautilusThumbnailInfo *currently_thumbnailing = NULL;
 
 static GnomeThumbnailFactory *thumbnail_factory = NULL;
 
+static int thumbnail_icon_size = 0;
+
 static gboolean
 get_file_mtime (const char *file_uri, time_t* mtime)
 {
@@ -115,21 +136,6 @@ get_file_mtime (const char *file_uri, time_t* mtime)
 	return TRUE;
 }
 
-/* GCompareFunc-style function for comparing NautilusThumbnailInfos.
- * Returns 0 if they refer to the same uri.
- */
-static int
-compare_thumbnail_info (gconstpointer a, gconstpointer b)
-{
-	NautilusThumbnailInfo *info_a;
-	NautilusThumbnailInfo *info_b;
-
-	info_a = (NautilusThumbnailInfo *)a;
-	info_b = (NautilusThumbnailInfo *)b;
-
-	return strcmp (info_a->image_uri, info_b->image_uri) != 0;
-}
-
 static void
 free_thumbnail_info (NautilusThumbnailInfo *info)
 {
@@ -290,10 +296,10 @@ thumbnail_loader_size_prepared (GdkPixbufLoader *loader,
 		args->base_size = size;                        
 	} else if (args->base_size == 0) {
 		if (args->is_thumbnail) {
-			args->base_size = 128 * NAUTILUS_ICON_SIZE_STANDARD / NAUTILUS_ICON_SIZE_THUMBNAIL;
+			args->base_size = 128 * NAUTILUS_ICON_SIZE_STANDARD / thumbnail_icon_size;
 		} else {
-			if (size > args->nominal_size * NAUTILUS_ICON_SIZE_THUMBNAIL / NAUTILUS_ICON_SIZE_STANDARD) {
-				args->base_size = size * NAUTILUS_ICON_SIZE_STANDARD / NAUTILUS_ICON_SIZE_THUMBNAIL;
+			if (size > args->nominal_size * thumbnail_icon_size / NAUTILUS_ICON_SIZE_STANDARD) {
+				args->base_size = size * NAUTILUS_ICON_SIZE_STANDARD / thumbnail_icon_size;
 			} else if (size > NAUTILUS_ICON_SIZE_STANDARD) {
 				args->base_size = args->nominal_size;
 			} else {
@@ -334,31 +340,24 @@ thumbnail_loader_area_prepared (GdkPixbufLoader *loader,
 	*args->scale_y_out = (double) gdk_pixbuf_get_height (pixbuf) / args->original_height;
 }
 
-/* routine to load an image from the passed-in path
- */
-GdkPixbuf *
-nautilus_thumbnail_load_image (const char *path,
-			       guint base_size,
-			       guint nominal_size,
-			       gboolean force_nominal,
-			       double *scale_x_out,
-			       double *scale_y_out)
+static GdkPixbuf *
+get_pixbuf_from_data (const unsigned char *buffer,
+		      gsize buflen,
+		      const char *path,
+		      guint base_size,
+		      guint nominal_size,
+		      gboolean force_nominal,
+		      double *scale_x_out,
+		      double *scale_y_out)
 {
-	guchar *buffer;
 	GdkPixbufLoader *loader;
 	GdkPixbuf *pixbuf;
-	GError *error;
-	gsize buflen;
 	ThumbnailLoadArgs args;
+	GError *error;
 
-	error = NULL;
-
-	if (!g_file_get_contents (path, (gchar **) &buffer, &buflen, &error)) {
-		g_message ("Failed to load %s into memory: %s", path, error->message);
-
-		g_error_free (error);
-
-		return NULL;
+	if (thumbnail_icon_size == 0) {
+		eel_preferences_add_auto_integer (NAUTILUS_PREFERENCES_ICON_VIEW_THUMBNAIL_SIZE,
+						  &thumbnail_icon_size);
 	}
 
 	loader = gdk_pixbuf_loader_new ();
@@ -376,17 +375,20 @@ nautilus_thumbnail_load_image (const char *path,
 	args.scale_x_out = scale_x_out;
 	args.scale_y_out = scale_y_out;
 
+	error = NULL;
+
 	if (!gdk_pixbuf_loader_write (loader, buffer, buflen, &error)) {
 		g_message ("Failed to write %s to thumbnail pixbuf loader: %s", path, error->message);
 
 		gdk_pixbuf_loader_close (loader, NULL);
 		g_object_unref (G_OBJECT (loader));
 		g_error_free (error);
-		g_free (buffer);
 
 		return NULL;
 	}
 
+	error = NULL;
+
 	if (!gdk_pixbuf_loader_close (loader, &error) ||
 	    /* Seems we have to check this even if it returned TRUE (#403255) */
 	    error != NULL) {
@@ -394,7 +396,6 @@ nautilus_thumbnail_load_image (const char *path,
 
 		g_object_unref (G_OBJECT (loader));
 		g_error_free (error);
-		g_free (buffer);
 
 		return NULL;
 	}
@@ -402,15 +403,125 @@ nautilus_thumbnail_load_image (const char *path,
 	pixbuf = g_object_ref (gdk_pixbuf_loader_get_pixbuf (loader));
 
 	g_object_unref (G_OBJECT (loader));
+
+	return pixbuf;
+}
+
+
+/* routine to load an image from the passed-in path
+ */
+GdkPixbuf *
+nautilus_thumbnail_load_image (const char *path,
+			       guint base_size,
+			       guint nominal_size,
+			       gboolean force_nominal,
+			       double *scale_x_out,
+			       double *scale_y_out)
+{
+	GdkPixbuf *pixbuf;
+	guchar *buffer;
+	gsize buflen;
+	GError *error;
+
+	error = NULL;
+	
+	if (!g_file_get_contents (path, (gchar **) &buffer, &buflen, &error)) {
+		g_message ("Failed to load %s into memory: %s", path, error->message);
+
+		g_error_free (error);
+
+		return NULL;
+	}
+
+	pixbuf = get_pixbuf_from_data (buffer, buflen, path,
+				       base_size, nominal_size, force_nominal,
+				       scale_x_out, scale_y_out);
+
 	g_free (buffer);
 
 	return pixbuf;
 }
 
+static void
+async_thumbnail_read_image (GnomeVFSResult result,
+			    GnomeVFSFileSize file_size,
+			    char *file_contents,
+			    gpointer callback_data)
+{
+	GdkPixbuf *pixbuf;
+	double scale_x, scale_y;
+
+	NautilusThumbnailAsyncLoadHandle *handle = callback_data;
+
+	pixbuf = NULL;
+	scale_x = scale_y = 1.0;
+
+	if (result == GNOME_VFS_OK) {
+		pixbuf = get_pixbuf_from_data (file_contents, file_size,
+					       handle->file_path,
+					       handle->base_size,
+					       handle->nominal_size,
+					       handle->force_nominal,
+					       &scale_x, &scale_y);
+	}
+
+	handle->load_func (handle,
+			   handle->file_path,
+			   pixbuf, scale_x, scale_y,
+			   handle->load_func_user_data);
+
+	gdk_pixbuf_unref (pixbuf);
+
+	g_free (handle->file_path);
+	g_free (handle);
+}
+
+NautilusThumbnailAsyncLoadHandle *
+nautilus_thumbnail_load_image_async (const char *path,
+				     guint base_size,
+				     guint nominal_size,
+				     gboolean force_nominal,
+				     NautilusThumbnailAsyncLoadFunc load_func,
+				     gpointer load_func_user_data)
+{
+	NautilusThumbnailAsyncLoadHandle *handle;
+	char *uri;
+
+	uri = gnome_vfs_get_uri_from_local_path (path);
+	if (uri == NULL) {
+		return NULL;
+	}
+
+	handle = g_new (NautilusThumbnailAsyncLoadHandle, 1);
+	handle->eel_read_handle =
+		eel_read_entire_file_async (uri, GNOME_VFS_PRIORITY_DEFAULT,
+					    (EelReadFileCallback) async_thumbnail_read_image,
+					   handle);
+	handle->file_path = g_strdup (path);
+	handle->base_size = base_size;
+	handle->nominal_size = nominal_size;
+	handle->force_nominal = force_nominal;
+	handle->load_func = load_func;
+	handle->load_func_user_data = load_func_user_data;
+
+	g_free (uri);
+
+	return handle;
+}
+
+void
+nautilus_thumbnail_load_image_cancel (NautilusThumbnailAsyncLoadHandle *handle)
+{
+	g_assert (handle != NULL);
+
+	eel_read_file_cancel (handle->eel_read_handle);
+	g_free (handle->file_path);
+	g_free (handle);
+}
+
 void
 nautilus_thumbnail_remove_from_queue (const char *file_uri)
 {
-	NautilusThumbnailInfo info;
 	GList *node;
 	
 #ifdef DEBUG_THUMBNAILS
@@ -422,15 +533,14 @@ nautilus_thumbnail_remove_from_queue (const char *file_uri)
 	 * MUTEX LOCKED
 	 *********************************/
 
-	info.image_uri = (char *)file_uri;
-	info.mime_type = NULL;
-	
-	node = g_list_find_custom ((GList*) thumbnails_to_make, &info,
-				   compare_thumbnail_info);
-
-	if (node && node->data != currently_thumbnailing) {
-		free_thumbnail_info (node->data);
-		thumbnails_to_make = g_list_delete_link ((GList *)thumbnails_to_make, node);
+	if (thumbnails_to_make_hash) {
+		node = g_hash_table_lookup (thumbnails_to_make_hash, file_uri);
+		
+		if (node && node->data != currently_thumbnailing) {
+			g_hash_table_remove (thumbnails_to_make_hash, file_uri);
+			free_thumbnail_info (node->data);
+			g_queue_delete_link ((GQueue *)&thumbnails_to_make, node);
+		}
 	}
 	
 	/*********************************
@@ -446,6 +556,7 @@ nautilus_thumbnail_remove_from_queue (const char *file_uri)
 void
 nautilus_thumbnail_remove_all_from_queue (void)
 {
+	NautilusThumbnailInfo *info;
 	GList *l, *next;
 	
 #ifdef DEBUG_THUMBNAILS
@@ -457,12 +568,15 @@ nautilus_thumbnail_remove_all_from_queue (void)
 	 * MUTEX LOCKED
 	 *********************************/
 
-	l = (GList *)thumbnails_to_make;
+	l = thumbnails_to_make.head;
 	while (l != NULL) {
+		info = l->data;
 		next = l->next;
-		if (l->data != currently_thumbnailing) {
-			free_thumbnail_info (l->data);
-			thumbnails_to_make = g_list_delete_link ((GList *)thumbnails_to_make, l);
+		if (info != currently_thumbnailing) {
+			g_hash_table_remove (thumbnails_to_make_hash, 
+					     info->image_uri);
+			free_thumbnail_info (info);
+			g_queue_delete_link ((GQueue *)&thumbnails_to_make, l);
 		}
 
 		l = next;
@@ -481,7 +595,6 @@ nautilus_thumbnail_remove_all_from_queue (void)
 void
 nautilus_thumbnail_prioritize (const char *file_uri)
 {
-	NautilusThumbnailInfo info;
 	GList *node;
 
 #ifdef DEBUG_THUMBNAILS
@@ -493,15 +606,13 @@ nautilus_thumbnail_prioritize (const char *file_uri)
 	 * MUTEX LOCKED
 	 *********************************/
 
-	info.image_uri = (char *)file_uri;
-	info.mime_type = NULL;
-	
-	node = g_list_find_custom ((GList*) thumbnails_to_make, &info,
-				   compare_thumbnail_info);
-
-	if (node && node->data != currently_thumbnailing) {
-		thumbnails_to_make = g_list_remove_link ((GList *)thumbnails_to_make, node);
-		thumbnails_to_make = g_list_concat (node, (GList *)thumbnails_to_make);
+	if (thumbnails_to_make_hash) {
+		node = g_hash_table_lookup (thumbnails_to_make_hash, file_uri);
+		
+		if (node && node->data != currently_thumbnailing) {
+			g_queue_unlink ((GQueue *)&thumbnails_to_make, node);
+			g_queue_push_head_link ((GQueue *)&thumbnails_to_make, node);
+		}
 	}
 	
 	/*********************************
@@ -554,7 +665,7 @@ nautilus_create_thumbnail (NautilusFile *file)
 	time_t file_mtime = 0;
 	NautilusThumbnailInfo *info;
 	NautilusThumbnailInfo *existing_info;
-	GList *existing;
+	GList *existing, *node;
 
 	nautilus_file_set_is_thumbnailing (file, TRUE);
 
@@ -579,28 +690,35 @@ nautilus_create_thumbnail (NautilusFile *file)
 	g_message ("(Main Thread) Locking mutex\n");
 #endif
 	pthread_mutex_lock (&thumbnails_mutex);
-
+	
 	/*********************************
 	 * MUTEX LOCKED
 	 *********************************/
 
+	if (thumbnails_to_make_hash == NULL) {
+		thumbnails_to_make_hash = g_hash_table_new (g_str_hash,
+							    g_str_equal);
+	}
+
 	/* Check if it is already in the list of thumbnails to make. */
-	existing = g_list_find_custom ((GList*) thumbnails_to_make, info,
-				       compare_thumbnail_info);
+	existing = g_hash_table_lookup (thumbnails_to_make_hash, info->image_uri);
 	if (existing == NULL) {
 		/* Add the thumbnail to the list. */
 #ifdef DEBUG_THUMBNAILS
 		g_message ("(Main Thread) Adding thumbnail: %s\n",
 			   info->image_uri);
 #endif
-		thumbnails_to_make = g_list_append ((GList*) thumbnails_to_make, info);
-		
+		g_queue_push_tail ((GQueue *)&thumbnails_to_make, info);
+		node = g_queue_peek_tail_link ((GQueue *)&thumbnails_to_make);
+		g_hash_table_insert (thumbnails_to_make_hash,
+				     info->image_uri,
+				     node);
 		/* If the thumbnail thread isn't running, and we haven't
 		   scheduled an idle function to start it up, do that now.
 		   We don't want to start it until all the other work is done,
 		   so the GUI will be updated as quickly as possible.*/
-		if (thumbnail_thread_is_running == FALSE
-		    && thumbnail_thread_starter_id == 0) {
+		if (thumbnail_thread_is_running == FALSE &&
+		    thumbnail_thread_starter_id == 0) {
 			thumbnail_thread_starter_id = g_idle_add_full (G_PRIORITY_LOW, thumbnail_thread_starter_cb, NULL, NULL);
 		}
 	} else {
@@ -612,8 +730,7 @@ nautilus_create_thumbnail (NautilusFile *file)
 		existing_info = existing->data;
 		existing_info->original_file_mtime = info->original_file_mtime;
 		free_thumbnail_info (info);
-	}
-	    
+	}   
 
 	/*********************************
 	 * MUTEX UNLOCKED
@@ -633,6 +750,7 @@ thumbnail_thread_start (gpointer data)
 	GdkPixbuf *pixbuf;
 	time_t current_orig_mtime = 0;
 	time_t current_time;
+	GList *node;
 
 	/* We loop until there are no more thumbails to make, at which point
 	   we exit the thread. */
@@ -656,15 +774,18 @@ thumbnail_thread_start (gpointer data)
 		if (currently_thumbnailing &&
 		    currently_thumbnailing->original_file_mtime == current_orig_mtime) {
 			g_assert (info == currently_thumbnailing);
-			free_thumbnail_info (currently_thumbnailing);
-			thumbnails_to_make = g_list_remove ((GList*) thumbnails_to_make, currently_thumbnailing);
+			node = g_hash_table_lookup (thumbnails_to_make_hash, info->image_uri);
+			g_assert (node != NULL);
+			g_hash_table_remove (thumbnails_to_make_hash, info->image_uri);
+			free_thumbnail_info (info);
+			g_queue_delete_link ((GQueue *)&thumbnails_to_make, node);
 		}
 		currently_thumbnailing = NULL;
 
 		/* If there are no more thumbnails to make, reset the
 		   thumbnail_thread_is_running flag, unlock the mutex, and
 		   exit the thread. */
-		if (thumbnails_to_make == NULL) {
+		if (g_queue_is_empty ((GQueue *)&thumbnails_to_make)) {
 #ifdef DEBUG_THUMBNAILS
 			g_message ("(Thumbnail Thread) Exiting\n");
 #endif
@@ -676,7 +797,7 @@ thumbnail_thread_start (gpointer data)
 		/* Get the next one to make. We leave it on the list until it
 		   is created so the main thread doesn't add it again while we
 		   are creating it. */
-		info = thumbnails_to_make->data;
+		info = g_queue_peek_head ((GQueue *)&thumbnails_to_make);
 		currently_thumbnailing = info;
 		current_orig_mtime = info->original_file_mtime;
 		/*********************************
