@@ -35,7 +35,7 @@
 #include <eel/eel-gtk-macros.h>
 #include "nautilus-trash-directory.h"
 #include <gtk/gtksignal.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <string.h>
 
 struct NautilusTrashFileDetails {
@@ -558,7 +558,8 @@ trash_file_get_item_count (NautilusFile *file,
 {
 	NautilusTrashFile *trash;
 	GList *node;
-	guint one_count, one_unreadable;
+	guint one_count;
+	int one_unreadable;
 	gboolean got_count;
 	
 	trash = NAUTILUS_TRASH_FILE (file);
