@@ -248,6 +248,7 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 	char *str;
 	GdkPixbuf *icon;
 	int icon_size;
+	guint emblem_size;
 	NautilusZoomLevel zoom_level;
 	char *modifier;
 	GList *emblem_icons;
@@ -333,14 +334,23 @@ fm_list_model_get_value (GtkTreeModel *tree_model, GtkTreeIter *iter, int column
 				}
 				nautilus_file_unref (parent_file);
 			}
-			emblem_icons = nautilus_icon_factory_get_emblem_icons_for_file (file, emblems_to_ignore);
+			zoom_level = fm_list_model_get_zoom_level_from_emblem_column_id (column);
+			icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
+			emblem_size = nautilus_icon_factory_get_emblem_size_for_icon_size (icon_size);
+			/* Special case default icon size here. This works semi-ok, since we
+			   only show one emblem for the list view anyway */
+			if (emblem_size == 0 && icon_size >= 24) {
+				emblem_size = 16;
+			}
+			emblem_icons = NULL;
+			if (emblem_size != 0) {
+				emblem_icons = nautilus_icon_factory_get_emblem_icons_for_file (file, emblems_to_ignore);
+			}
 			eel_string_list_free (emblems_to_ignore);
 
 			if (emblem_icons != NULL) {
-				zoom_level = fm_list_model_get_zoom_level_from_emblem_column_id (column);
-				icon_size = nautilus_get_icon_size_for_zoom_level (zoom_level);
 				icon = nautilus_icon_factory_get_pixbuf_for_icon (
-					emblem_icons->data, NULL, icon_size,
+					emblem_icons->data, NULL, emblem_size,
 					NULL, NULL, TRUE, FALSE, NULL);
 				eel_g_list_free_deep (emblem_icons);
 		
