@@ -49,7 +49,6 @@
 #include <eel/eel-gtk-macros.h>
 #include <eel/eel-stock-dialogs.h>
 #include <eel/eel-string.h>
-#include <eel/eel-vfs-extensions.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtkmain.h>
@@ -62,13 +61,10 @@
 #include <gtk/gtktreemodel.h>
 #include <gtk/gtkliststore.h>
 #include <glib/gi18n.h>
-#include <libgnome/gnome-macros.h>
-#include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-messagebox.h>
+#ifdef HAVE_X11_XF86KEYSYM_H
+#include <X11/XF86keysym.h>
+#endif
 #include <libgnomeui/gnome-uidefs.h>
-#include <libgnomeui/gnome-window-icon.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
@@ -136,8 +132,21 @@ static void search_bar_cancel_callback               (GtkWidget                *
 
 static void nautilus_navigation_window_show_location_bar_temporarily (NautilusNavigationWindow *window);
 
-GNOME_CLASS_BOILERPLATE (NautilusNavigationWindow, nautilus_navigation_window,
-			 NautilusWindow, NAUTILUS_TYPE_WINDOW)
+static void view_as_menu_switch_views_callback	     (GtkComboBox              *combo_box,
+						      NautilusWindow           *window);
+
+G_DEFINE_TYPE (NautilusNavigationWindow, nautilus_navigation_window, NAUTILUS_TYPE_WINDOW)
+#define parent_class nautilus_navigation_window_parent_class
+
+static const struct {
+	unsigned int keyval;
+	const char *action;
+} extra_navigation_window_keybindings [] = {
+#ifdef HAVE_X11_XF86KEYSYM_H
+	{ XF86XK_Back,		NAUTILUS_ACTION_BACK },
+	{ XF86XK_Forward,	NAUTILUS_ACTION_FORWARD }
+#endif
+};
 
 static void
 location_button_toggled_cb (GtkToggleButton *toggle,
@@ -170,11 +179,12 @@ location_button_create (NautilusNavigationWindow *window)
 
 	button = g_object_new (GTK_TYPE_TOGGLE_BUTTON,
 			       "image", image,
+			       "focus-on-click", FALSE,
 			       "active", location_button_should_be_active (window),
 			       NULL);
 
-	gtk_tooltips_set_tip (window->details->tooltips, button,
-			      _("Toggle between button and text-based location bar"), NULL);
+	gtk_widget_set_tooltip_text (button,
+				     _("Toggle between button and text-based location bar"));
 
 	g_signal_connect (button, "toggled",
 			  G_CALLBACK (location_button_toggled_cb), window);
@@ -182,7 +192,7 @@ location_button_create (NautilusNavigationWindow *window)
 }
 
 static void
-nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
+nautilus_navigation_window_init (NautilusNavigationWindow *window)
 {
 	GtkUIManager *ui_manager;
 	GtkWidget *toolbar;
@@ -190,12 +200,8 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 	GtkWidget *view_as_menu_vbox;
 	GtkToolItem *item;
 	GtkWidget *hbox, *vbox, *eventbox, *extras_vbox;
-	
-	window->details = g_new0 (NautilusNavigationWindowDetails, 1);
 
-	window->details->tooltips = gtk_tooltips_new ();
-	g_object_ref (window->details->tooltips);
-	gtk_object_sink (GTK_OBJECT (window->details->tooltips));
+	window->details = G_TYPE_INSTANCE_GET_PRIVATE (window, NAUTILUS_TYPE_NAVIGATION_WINDOW, NautilusNavigationWindowDetails);
 
 	window->details->content_paned = nautilus_horizontal_splitter_new ();
 	gtk_table_attach (GTK_TABLE (NAUTILUS_WINDOW (window)->details->table),
@@ -310,8 +316,11 @@ nautilus_navigation_window_instance_init (NautilusNavigationWindow *window)
 			    item, -1);
 	
 	window->view_as_combo_box = gtk_combo_box_new_text ();
+	gtk_combo_box_set_focus_on_click (GTK_COMBO_BOX (window->view_as_combo_box), FALSE);
 	gtk_box_pack_end (GTK_BOX (view_as_menu_vbox), window->view_as_combo_box, TRUE, FALSE, 0);
 	gtk_widget_show (window->view_as_combo_box);
+	g_signal_connect_object (window->view_as_combo_box, "changed",
+				 G_CALLBACK (view_as_menu_switch_views_callback), window, 0);
 
 	/* Allocate the zoom control and place on the right next to the menu.
 	 * It gets shown later, if the view-frame contains something zoomable.
@@ -637,6 +646,36 @@ nautilus_navigation_window_state_event (GtkWidget *widget,
 	return FALSE;
 }
 
+static gboolean
+nautilus_navigation_window_key_press_event (GtkWidget *widget,
+					    GdkEventKey *event)
+{
+	NautilusNavigationWindow *window;
+	int i;
+
+	window = NAUTILUS_NAVIGATION_WINDOW (widget);
+
+	for (i = 0; i < G_N_ELEMENTS (extra_navigation_window_keybindings); i++) {
+		if (extra_navigation_window_keybindings[i].keyval == event->keyval) {
+			GtkAction *action;
+
+			action = gtk_action_group_get_action (window->details->navigation_action_group,
+							      extra_navigation_window_keybindings[i].action);
+			g_assert (action != NULL);
+
+			g_assert (action != NULL);
+			if (gtk_action_is_sensitive (action)) {
+				gtk_action_activate (action);
+				return TRUE;
+			}
+
+			break;
+		}
+	}
+
+	return GTK_WIDGET_CLASS (nautilus_navigation_window_parent_class)->key_press_event (widget, event);
+}
+
 static void
 nautilus_navigation_window_destroy (GtkObject *object)
 {
@@ -655,11 +694,6 @@ nautilus_navigation_window_destroy (GtkObject *object)
 
 	window->details->content_paned = NULL;
 
-	if (window->details->tooltips) {
-		g_object_unref (window->details->tooltips);
-		window->details->tooltips = NULL;
-	}
-	
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
@@ -674,8 +708,6 @@ nautilus_navigation_window_finalize (GObject *object)
 	nautilus_navigation_window_clear_back_list (window);
 	nautilus_navigation_window_clear_forward_list (window);
 
- 	g_free (window->details);
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -764,18 +796,6 @@ nautilus_navigation_window_go_forward (NautilusNavigationWindow *window)
 	nautilus_navigation_window_back_or_forward (window, FALSE, 0);
 }
 
-void
-nautilus_navigation_window_go_home (NautilusNavigationWindow *window)
-{
-	char *home_uri;
-
-	home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
-	
-	g_assert (home_uri != NULL);
-	nautilus_window_go_to (NAUTILUS_WINDOW (window), home_uri);
-	g_free (home_uri);
-}
-
 void
 nautilus_navigation_window_allow_back (NautilusNavigationWindow *window, gboolean allow)
 {
@@ -825,8 +845,10 @@ view_as_menu_switch_views_callback (GtkComboBox *combo_box, NautilusWindow *wind
 	g_assert (NAUTILUS_IS_WINDOW (window));
 
 	active = gtk_combo_box_get_active (combo_box);
-	
-	if (active < GPOINTER_TO_INT (g_object_get_data (G_OBJECT (combo_box), "num viewers"))  ) {
+
+	if (active < 0) {
+		return;
+	} else if (active < GPOINTER_TO_INT (g_object_get_data (G_OBJECT (combo_box), "num viewers"))) {
 		activate_nth_short_list_item (window, active);
 	} else {
 		activate_extra_viewer (window);
@@ -877,9 +899,6 @@ load_view_as_menu (NautilusWindow *window)
 		selected_index = index;
 	}
 
-	g_signal_connect (GTK_COMBO_BOX (NAUTILUS_NAVIGATION_WINDOW (window)->view_as_combo_box), 
-			  "changed", G_CALLBACK (view_as_menu_switch_views_callback), window);
-
 	gtk_combo_box_set_active (GTK_COMBO_BOX (NAUTILUS_NAVIGATION_WINDOW (window)->view_as_combo_box), selected_index);
 }
 
@@ -1200,7 +1219,6 @@ add_sidebar_panels (NautilusNavigationWindow *window)
 	GList *providers;
 	GList *p;
 	NautilusSidebar *sidebar_panel;
-	NautilusSidebarProvider *provider;
 
 	g_assert (NAUTILUS_IS_NAVIGATION_WINDOW (window));
 
@@ -1211,6 +1229,8 @@ add_sidebar_panels (NautilusNavigationWindow *window)
  	providers = nautilus_module_get_extensions_for_type (NAUTILUS_TYPE_SIDEBAR_PROVIDER);
 	
 	for (p = providers; p != NULL; p = p->next) {
+		NautilusSidebarProvider *provider;
+
 		provider = NAUTILUS_SIDEBAR_PROVIDER (p->data);
 		
 		sidebar_panel = nautilus_sidebar_provider_create (provider,
@@ -1220,7 +1240,9 @@ add_sidebar_panels (NautilusNavigationWindow *window)
 		
 		g_object_unref (sidebar_panel);
 	}
-	
+
+	nautilus_module_extension_list_free (providers);
+
 	current = nautilus_side_pane_get_current_panel (window->sidebar);
 	set_current_side_panel
 		(window, 
@@ -1550,6 +1572,7 @@ nautilus_navigation_window_class_init (NautilusNavigationWindowClass *class)
 	GTK_WIDGET_CLASS (class)->show = nautilus_navigation_window_show;
 	GTK_WIDGET_CLASS (class)->unrealize = nautilus_navigation_window_unrealize;
 	GTK_WIDGET_CLASS (class)->window_state_event = nautilus_navigation_window_state_event;
+	GTK_WIDGET_CLASS (class)->key_press_event = nautilus_navigation_window_key_press_event;
 	NAUTILUS_WINDOW_CLASS (class)->load_view_as_menu = real_load_view_as_menu;
 	NAUTILUS_WINDOW_CLASS (class)->set_content_view_widget = real_set_content_view_widget;
 	NAUTILUS_WINDOW_CLASS (class)->set_throbber_active = real_set_throbber_active;
@@ -1559,4 +1582,6 @@ nautilus_navigation_window_class_init (NautilusNavigationWindowClass *class)
 	NAUTILUS_WINDOW_CLASS (class)->get_icon_name = real_get_icon_name;
 	NAUTILUS_WINDOW_CLASS (class)->get_default_size = real_get_default_size;
 	NAUTILUS_WINDOW_CLASS (class)->close = real_window_close;
+
+	g_type_class_add_private (G_OBJECT_CLASS (class), sizeof (NautilusNavigationWindowDetails));
 }
