@@ -43,7 +43,7 @@
 #include <gtk/gtklabel.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkwidget.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-types.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
@@ -89,7 +89,6 @@ struct NautilusSidebarTitleDetails {
 	GtkWidget		*title_label;
 	GtkWidget		*more_info_label;
 	GtkWidget		*emblem_box;
-	GtkWidget		*notes;
 
 	guint                    best_icon_size;
 
@@ -168,11 +167,6 @@ nautilus_sidebar_title_init (NautilusSidebarTitle *sidebar_title)
 	gtk_widget_show (sidebar_title->details->emblem_box);
 	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->emblem_box, 0, 0, 0);
 
-	sidebar_title->details->notes = GTK_WIDGET (gtk_label_new (NULL));
-	gtk_label_set_line_wrap (GTK_LABEL (sidebar_title->details->notes), TRUE);
-	gtk_widget_show (sidebar_title->details->notes);
-	gtk_box_pack_start (GTK_BOX (sidebar_title), sidebar_title->details->notes, 0, 0, 0);
-
 	sidebar_title->details->best_icon_size = get_best_icon_size (sidebar_title);
 	/* Keep track of changes in graphics trade offs */
 	update_all (sidebar_title);
@@ -515,7 +509,7 @@ update_emblems (NautilusSidebarTitle *sidebar_title)
 	for (p = icons; p != NULL; p = p->next) {
 		pixbuf = nautilus_icon_factory_get_pixbuf_for_icon
 			(p->data, NULL,
-			 NAUTILUS_ICON_SIZE_STANDARD,
+			 nautilus_icon_factory_get_emblem_size_for_icon_size (NAUTILUS_ICON_SIZE_STANDARD),
 			 NULL, NULL,
 			 FALSE, FALSE, NULL);
 		if (pixbuf != NULL) {
@@ -527,18 +521,6 @@ update_emblems (NautilusSidebarTitle *sidebar_title)
 	eel_g_list_free_deep (icons);
 }
 
-static void
-update_notes (NautilusSidebarTitle *sidebar_title)
-{
-	char *text;
-	
-	text = nautilus_file_get_metadata (sidebar_title->details->file,
-					   NAUTILUS_METADATA_KEY_NOTES,
-					   NULL);
-	gtk_label_set_text (GTK_LABEL (sidebar_title->details->notes), text);
-	g_free (text);
-}
-
 /* return the filename text */
 char *
 nautilus_sidebar_title_get_text (NautilusSidebarTitle *sidebar_title)
@@ -607,7 +589,6 @@ update_all (NautilusSidebarTitle *sidebar_title)
 	update_more_info (sidebar_title);
 	
 	update_emblems (sidebar_title);
-	update_notes (sidebar_title);
 
 	/* Redo monitor once the count is ready. */
 	if (!sidebar_title->details->monitoring_count && item_count_ready (sidebar_title)) {
@@ -689,7 +670,6 @@ sidebar_title_create_title_label (void)
 	gtk_label_set_justify (GTK_LABEL (title_label), GTK_JUSTIFY_CENTER);
 	gtk_label_set_selectable (GTK_LABEL (title_label), TRUE);
 	gtk_label_set_ellipsize (GTK_LABEL (title_label), PANGO_ELLIPSIZE_END);
-	gtk_misc_set_alignment (GTK_MISC (title_label), 0., 0.5);
 
 	return title_label;
 }
@@ -704,7 +684,6 @@ sidebar_title_create_more_info_label (void)
 	gtk_label_set_justify (GTK_LABEL (more_info_label), GTK_JUSTIFY_CENTER);
 	gtk_label_set_selectable (GTK_LABEL (more_info_label), TRUE);
 	gtk_label_set_ellipsize (GTK_LABEL (more_info_label), PANGO_ELLIPSIZE_END);
-	gtk_misc_set_alignment (GTK_MISC (more_info_label), 0., 0.5);
 	
 	return more_info_label;
 }
